@@ -10,12 +10,12 @@ from typing import Optional, Tuple, Union, cast
 import aiohttp
 import trafaret as t
 from aiohttp import web
-from aiohttp_session import STORAGE_KEY, get_session
 from Crypto.Cipher import AES
 from Crypto.Util.Padding import unpad
 
 from ai.backend.client.exceptions import BackendAPIError, BackendClientError
 from ai.backend.client.request import Request
+from ai.backend.common.web.session import STORAGE_KEY, get_session
 
 from .auth import get_anonymous_session, get_api_session
 from .logging import BraceStyleAdapter
