@@ -14,259 +14,30 @@ from pprint import pprint
 from typing import Any, AsyncIterator, MutableMapping, Tuple
 
 import aiohttp_cors
-import aioredis
 import aiotools
 import click
 import jinja2
-import pkg_resources
-import toml
+import tomli
 import uvloop
 import yarl
 from aiohttp import web
-from aiohttp_session import get_session
-from aiohttp_session import setup as setup_session
-from aiohttp_session.redis_storage import RedisStorage
+from redis.asyncio import Redis
 from setproctitle import setproctitle
 
 from ai.backend.client.config import APIConfig
 from ai.backend.client.exceptions import BackendAPIError, BackendClientError
 from ai.backend.client.session import AsyncSession as APISession
+from ai.backend.common.web.session import get_session
+from ai.backend.common.web.session import setup as setup_session
+from ai.backend.common.web.session.redis_storage import RedisStorage
 
 from . import __version__, user_agent
+from .config import config_iv
 from .logging import BraceStyleAdapter
-from .proxy import (
-    decrypt_payload,
-    extra_config_headers,
-    web_handler,
-    web_plugin_handler,
-    websocket_handler,
-)
+from .proxy import decrypt_payload, web_handler, web_plugin_handler, websocket_handler
+from .template import toml_scalar
 
 log = BraceStyleAdapter(logging.getLogger("ai.backend.web.server"))
-static_path = Path(pkg_resources.resource_filename("ai.backend.web", "static")).resolve()
-assert static_path.is_dir()
-
-
-console_config_ini_template = jinja2.Template(
-    """[general]
-apiEndpoint = {{endpoint_url}}
-apiEndpointText = {{endpoint_text}}
-{% if default_environment %}
-defaultSessionEnvironment = "{{default_environment}}"
-{% endif %}
-siteDescription = {{site_description}}
-connectionMode = "SESSION"
-
-[wsproxy]
-proxyURL = {{proxy_url}}/
-proxyBaseURL =
-proxyListenIP =
-"""
-)
-
-console_config_toml_template = jinja2.Template(
-    """[general]
-apiEndpoint = "{{endpoint_url}}"
-apiEndpointText = "{{endpoint_text}}"
-{% if default_environment %}
-defaultSessionEnvironment = "{{default_environment}}"
-{% endif %}
-{% if default_import_environment %}
-defaultImportEnvironment = "{{default_import_environment}}"
-{% endif %}
-siteDescription = "{{site_description}}"
-connectionMode = "SESSION"
-signupSupport = {{signup_support}}
-allowChangeSigninMode = {{allow_change_signin_mode}}
-allowAnonymousChangePassword = {{allow_anonymous_change_password}}
-allowProjectResourceMonitor = {{allow_project_resource_monitor}}
-allowManualImageNameForSession = {{allow_manual_image_name_for_session}}
-allowSignupWithoutConfirmation = {{allow_signup_without_confirmation}}
-autoLogout = {{auto_logout}}
-debug = {{webui_debug}}
-maskUserInfo = {{mask_user_info}}
-
-[resources]
-openPortToPublic = {{open_port_to_public}}
-maxCPUCoresPerContainer = {{max_cpu_cores_per_container}}
-maxMemoryPerContainer = {{max_memory_per_container}}
-maxCUDADevicesPerContainer = {{max_cuda_devices_per_container}}
-maxCUDASharesPerContainer = {{max_cuda_shares_per_container}}
-maxShmPerContainer = {{max_shm_per_container}}
-maxFileUploadSize = {{max_file_upload_size}}
-
-[environments]
-{% if environment_allowlist %}
-allowlist = "{{environment_allowlist}}"
-{% endif %}
-
-[menu]
-{% if menu_blocklist %}
-blocklist = "{{menu_blocklist}}"
-{% endif %}
-
-{% if console_menu_plugins %}
-[plugin]
-page = "{{console_menu_plugins}}"
-
-{% endif %}
-[wsproxy]
-proxyURL = "{{proxy_url}}/"
-#proxyBaseURL =
-#proxyListenIP =
-
-[license]
-edition = "{{license_edition}}"
-validSince = "{{license_valid_since}}"
-validUntil = "{{license_valid_until}}"
-"""
-)
-
-
-async def static_handler(request: web.Request) -> web.StreamResponse:
-    request_path = request.match_info["path"]
-    file_path = (static_path / request_path).resolve()
-    try:
-        file_path.relative_to(static_path)
-    except (ValueError, FileNotFoundError):
-        return web.HTTPNotFound(
-            text=json.dumps(
-                {
-                    "type": "https://api.backend.ai/probs/generic-not-found",
-                    "title": "Not Found",
-                }
-            ),
-            content_type="application/problem+json",
-        )
-    if file_path.is_file():
-        return header_handler(web.FileResponse(file_path), request_path)
-    return web.HTTPNotFound(
-        text=json.dumps(
-            {
-                "type": "https://api.backend.ai/probs/generic-not-found",
-                "title": "Not Found",
-            }
-        ),
-        content_type="application/problem+json",
-    )
-
-
-async def console_handler(request: web.Request) -> web.StreamResponse:
-    request_path = request.match_info["path"]
-    file_path = (static_path / request_path).resolve()
-    config = request.app["config"]
-    scheme = config["service"].get("force-endpoint-protocol")
-    if scheme is None:
-        scheme = request.scheme
-
-    if request_path == "config.ini":
-        config_content = console_config_ini_template.render(
-            **{
-                "endpoint_url": f"{scheme}://{request.host}",  # must be absolute
-                "endpoint_text": config["api"]["text"],
-                "site_description": config["ui"]["brand"],
-                "default_environment": config["ui"].get("default_environment"),
-                "proxy_url": config["service"]["wsproxy"]["url"],
-            }
-        )
-        return web.Response(text=config_content)
-
-    if request_path == "config.toml":
-        if "license" in config:
-            license_edition = config["license"].get("edition", "Open Source")
-            license_valid_since = config["license"].get("valid_since", "")
-            license_valid_until = config["license"].get("valid_until", "")
-        else:
-            license_edition = "Open Source"
-            license_valid_since = ""
-            license_valid_until = ""
-        if "resources" in config:
-            open_port_to_public = (
-                "true" if config["resources"].get("open_port_to_public") else "false"
-            )
-            max_cpu_cores_per_container = config["resources"].get("max_cpu_cores_per_container", 64)
-            max_memory_per_container = config["resources"].get("max_memory_per_container", 64)
-            max_cuda_devices_per_container = config["resources"].get(
-                "max_cuda_devices_per_container", 16
-            )
-            max_cuda_shares_per_container = config["resources"].get(
-                "max_cuda_shares_per_container", 16
-            )
-            max_shm_per_container = config["resources"].get("max_shm_per_container", 2)
-            max_file_upload_size = config["resources"].get("max_file_upload_size", 4294967296)
-        else:
-            open_port_to_public = "false"
-            max_cpu_cores_per_container = 64
-            max_memory_per_container = 64
-            max_cuda_devices_per_container = 16
-            max_cuda_shares_per_container = 16
-            max_shm_per_container = 2
-            max_file_upload_size = 4294967296
-        if "plugin" in config:
-            console_menu_plugins = config["plugin"].get("page", "")
-        else:
-            console_menu_plugins = False
-        config_content = console_config_toml_template.render(
-            **{
-                "endpoint_url": f"{scheme}://{request.host}",  # must be absolute
-                "endpoint_text": config["api"]["text"],
-                "site_description": config["ui"]["brand"],
-                "default_environment": config["ui"].get("default_environment"),
-                "default_import_environment": config["ui"].get("default_import_environment"),
-                "proxy_url": config["service"]["wsproxy"]["url"],
-                "signup_support": "true" if config["service"]["enable_signup"] else "false",
-                "allow_change_signin_mode": "true"
-                if config["service"].get("allow_change_signin_mode")
-                else "false",
-                "allow_anonymous_change_password": "true"
-                if config["service"].get("allow_anonymous_change_password")
-                else "false",
-                "allow_project_resource_monitor": "true"
-                if config["service"]["allow_project_resource_monitor"]
-                else "false",
-                "allow_manual_image_name_for_session": "true"
-                if config["service"].get("allow_manual_image_name_for_session")
-                else "false",
-                "allow_signup_without_confirmation": "true"
-                if config["service"].get("allow_signup_without_confirmation")
-                else "false",
-                "webui_debug": "true" if config["service"].get("webui_debug") else "false",
-                "auto_logout": "true" if config["session"].get("auto_logout") else "false",
-                "mask_user_info": "true" if config["service"].get("mask_user_info") else "false",
-                "open_port_to_public": open_port_to_public,
-                "max_cpu_cores_per_container": max_cpu_cores_per_container,
-                "max_memory_per_container": max_memory_per_container,
-                "max_cuda_devices_per_container": max_cuda_devices_per_container,
-                "max_cuda_shares_per_container": max_cuda_shares_per_container,
-                "max_shm_per_container": max_shm_per_container,
-                "max_file_upload_size": max_file_upload_size,
-                "environment_allowlist": config["environments"].get("allowlist", ""),
-                "menu_blocklist": config["ui"].get("menu_blocklist", ""),
-                "console_menu_plugins": console_menu_plugins,
-                "license_edition": license_edition,
-                "license_valid_since": license_valid_since,
-                "license_valid_until": license_valid_until,
-            }
-        )
-        return web.Response(text=config_content)
-    # SECURITY: only allow reading files under static_path
-    try:
-        file_path.relative_to(static_path)
-    except (ValueError, FileNotFoundError):
-        return web.HTTPNotFound(
-            text=json.dumps(
-                {
-                    "type": "https://api.backend.ai/probs/generic-not-found",
-                    "title": "Not Found",
-                }
-            ),
-            content_type="application/problem+json",
-        )
-    if file_path.is_file():
-        return header_handler(web.FileResponse(file_path), request_path)
-
-    return header_handler(web.FileResponse(static_path / "index.html"), "index.html")
-
 
 cache_patterns = {
     r"\.(?:manifest|appcache|html?|xml|json|ini|toml)$": {
@@ -288,7 +59,7 @@ cache_patterns = {
 _cache_patterns = {re.compile(k): v for k, v in cache_patterns.items()}
 
 
-def header_handler(response: web.StreamResponse, path: str) -> web.StreamResponse:
+def apply_cache_headers(response: web.StreamResponse, path: str) -> web.StreamResponse:
     for regex, headers in _cache_patterns.items():
         mo = regex.search(path)
         if mo is not None:
@@ -297,6 +68,91 @@ def header_handler(response: web.StreamResponse, path: str) -> web.StreamRespons
     return response
 
 
+async def static_handler(request: web.Request) -> web.StreamResponse:
+    request_path = request.match_info["path"]
+    static_path = request.app["config"]["service"]["static_path"]
+    file_path = (static_path / request_path).resolve()
+    try:
+        file_path.relative_to(static_path)
+    except (ValueError, FileNotFoundError):
+        return web.HTTPNotFound(
+            text=json.dumps(
+                {
+                    "type": "https://api.backend.ai/probs/generic-not-found",
+                    "title": "Not Found",
+                }
+            ),
+            content_type="application/problem+json",
+        )
+    if file_path.is_file():
+        return apply_cache_headers(web.FileResponse(file_path), request_path)
+    return web.HTTPNotFound(
+        text=json.dumps(
+            {
+                "type": "https://api.backend.ai/probs/generic-not-found",
+                "title": "Not Found",
+            }
+        ),
+        content_type="application/problem+json",
+    )
+
+
+async def config_ini_handler(request: web.Request) -> web.Response:
+    config = request.app["config"]
+    scheme = config["service"]["force_endpoint_protocol"]
+    if scheme is None:
+        scheme = request.scheme
+    j2env: jinja2.Environment = request.app["j2env"]
+    tpl = j2env.get_template("config_ini.toml.j2")
+    config_content = tpl.render(
+        {
+            "endpoint_url": f"{scheme}://{request.host}",  # must be absolute
+            "config": config,
+        }
+    )
+    return web.Response(text=config_content, content_type="text/plain")
+
+
+async def config_toml_handler(request: web.Request) -> web.Response:
+    config = request.app["config"]
+    scheme = config["service"]["force_endpoint_protocol"]
+    if scheme is None:
+        scheme = request.scheme
+    j2env: jinja2.Environment = request.app["j2env"]
+    tpl = j2env.get_template("config.toml.j2")
+    config_content = tpl.render(
+        {
+            "endpoint_url": f"{scheme}://{request.host}",  # must be absolute
+            "config": config,
+        }
+    )
+    return web.Response(text=config_content, content_type="text/plain")
+
+
+async def console_handler(request: web.Request) -> web.StreamResponse:
+    request_path = request.match_info["path"]
+    config = request.app["config"]
+    static_path = config["service"]["static_path"]
+    file_path = (static_path / request_path).resolve()
+    # SECURITY: only allow reading files under static_path
+    try:
+        file_path.relative_to(static_path)
+    except (ValueError, FileNotFoundError):
+        return web.HTTPNotFound(
+            text=json.dumps(
+                {
+                    "type": "https://api.backend.ai/probs/generic-not-found",
+                    "title": "Not Found",
+                }
+            ),
+            content_type="application/problem+json",
+        )
+    if file_path.is_file():
+        return apply_cache_headers(web.FileResponse(file_path), request_path)
+    # Fallback to index.html to support the URL routing for single-page application.
+    return apply_cache_headers(web.FileResponse(static_path / "index.html"), "index.html")
+
+
 async def login_check_handler(request: web.Request) -> web.Response:
     session = await get_session(request)
     authenticated = bool(session.get("authenticated", False))
@@ -330,13 +186,11 @@ async def login_handler(request: web.Request) -> web.Response:
             ),
             content_type="application/problem+json",
         )
-    request_headers = extra_config_headers.check(request.headers)
-    secure_context = request_headers.get("X-BackendAI-Encoded", None)
-    if secure_context:
-        creds = await decrypt_payload(request)
-        creds = json.loads(creds)
-    else:
-        creds = await request.json()
+    try:
+        creds = json.loads(request["payload"])
+    except json.JSONDecodeError as e:
+        log.error("Login: JSON decoding error: {}", e)
+        creds = {}
     if "username" not in creds or not creds["username"]:
         return web.HTTPBadRequest(
             text=json.dumps(
@@ -361,73 +215,73 @@ async def login_handler(request: web.Request) -> web.Response:
         "authenticated": False,
         "data": None,
     }
-    try:
 
-        async def _get_login_history():
-            login_history = await request.app["redis"].get(
-                f'login_history_{creds["username"]}',
-            )
-            if not login_history:
-                login_history = {
-                    "last_login_attempt": 0,
-                    "login_fail_count": 0,
-                }
-            else:
-                login_history = json.loads(login_history)
-            if login_history["last_login_attempt"] < 0:
-                login_history["last_login_attempt"] = 0
-            if login_history["login_fail_count"] < 0:
-                login_history["login_fail_count"] = 0
-            return login_history
+    async def _get_login_history():
+        login_history = await request.app["redis"].get(
+            f'login_history_{creds["username"]}',
+        )
+        if not login_history:
+            login_history = {
+                "last_login_attempt": 0,
+                "login_fail_count": 0,
+            }
+        else:
+            login_history = json.loads(login_history)
+        if login_history["last_login_attempt"] < 0:
+            login_history["last_login_attempt"] = 0
+        if login_history["login_fail_count"] < 0:
+            login_history["login_fail_count"] = 0
+        return login_history
 
-        async def _set_login_history(last_login_attempt, login_fail_count):
-            """
-            Set login history per email (not in browser session).
-            """
-            key = f'login_history_{creds["username"]}'
-            value = json.dumps(
+    async def _set_login_history(last_login_attempt, login_fail_count):
+        """
+        Set login history per email (not in browser session).
+        """
+        key = f'login_history_{creds["username"]}'
+        value = json.dumps(
+            {
+                "last_login_attempt": last_login_attempt,
+                "login_fail_count": login_fail_count,
+            }
+        )
+        await request.app["redis"].set(key, value)
+
+    # Block login if there are too many consecutive failed login attempts.
+    BLOCK_TIME = config["session"]["login_block_time"]
+    ALLOWED_FAIL_COUNT = config["session"]["login_allowed_fail_count"]
+    login_time = time.time()
+    login_history = await _get_login_history()
+    last_login_attempt = login_history.get("last_login_attempt", 0)
+    login_fail_count = login_history.get("login_fail_count", 0)
+    if login_time - last_login_attempt > BLOCK_TIME:
+        # If last attempt is far past, allow login again.
+        login_fail_count = 0
+    last_login_attempt = login_time
+    if login_fail_count >= ALLOWED_FAIL_COUNT:
+        log.info(
+            "Too many consecutive login attempts for {}: {}",
+            creds["username"],
+            login_fail_count,
+        )
+        await _set_login_history(last_login_attempt, login_fail_count)
+        return web.HTTPTooManyRequests(
+            text=json.dumps(
                 {
-                    "last_login_attempt": last_login_attempt,
-                    "login_fail_count": login_fail_count,
+                    "type": "https://api.backend.ai/probs/too-many-requests",
+                    "title": "Too many failed login attempts",
                 }
-            )
-            await request.app["redis"].set(key, value)
-
-        # Block login if there are too many consecutive failed login attempts.
-        BLOCK_TIME = config["session"].get("login_block_time", 1200)
-        ALLOWED_FAIL_COUNT = config["session"].get("login_allowed_fail_count", 10)
-        login_time = time.time()
-        login_history = await _get_login_history()
-        last_login_attempt = login_history.get("last_login_attempt", 0)
-        login_fail_count = login_history.get("login_fail_count", 0)
-        if login_time - last_login_attempt > BLOCK_TIME:
-            # If last attempt is far past, allow login again.
-            login_fail_count = 0
-        last_login_attempt = login_time
-        if login_fail_count >= ALLOWED_FAIL_COUNT:
-            log.info(
-                "Too many consecutive login attempts for {}: {}",
-                creds["username"],
-                login_fail_count,
-            )
-            await _set_login_history(last_login_attempt, login_fail_count)
-            return web.HTTPTooManyRequests(
-                text=json.dumps(
-                    {
-                        "type": "https://api.backend.ai/probs/too-many-requests",
-                        "title": "Too many failed login attempts",
-                    }
-                ),
-                content_type="application/problem+json",
-            )
+            ),
+            content_type="application/problem+json",
+        )
 
+    try:
         anon_api_config = APIConfig(
             domain=config["api"]["domain"],
-            endpoint=config["api"]["endpoint"],
+            endpoint=config["api"]["endpoint"][0],
             access_key="",
             secret_key="",  # anonymous session
             user_agent=user_agent,
-            skip_sslcert_validation=not config["api"].get("ssl-verify", True),
+            skip_sslcert_validation=not config["api"]["ssl_verify"],
         )
         assert anon_api_config.is_anonymous
         async with APISession(config=anon_api_config) as api_session:
@@ -482,7 +336,7 @@ async def logout_handler(request: web.Request) -> web.Response:
     return web.Response(status=201)
 
 
-async def webserver_healthcheck(request: web.Request) -> web.Response:
+async def webserver_healthcheck(_: web.Request) -> web.Response:
     result = {
         "version": __version__,
         "details": "Success",
@@ -507,17 +361,7 @@ async def token_login_handler(request: web.Request) -> web.Response:
         )
 
     # Check if auth token is delivered through cookie.
-    auth_token_name = config["api"].get("auth_token_name")
-    if not auth_token_name:
-        return web.HTTPBadRequest(
-            text=json.dumps(
-                {
-                    "type": "https://api.backend.ai/probs/invalid-api-params",
-                    "title": "Auth token name is not defined",
-                }
-            ),
-            content_type="application/problem+json",
-        )
+    auth_token_name = config["api"]["auth_token_name"]
     auth_token = request.cookies.get(auth_token_name)
     if not auth_token:
         return web.HTTPBadRequest(
@@ -540,11 +384,11 @@ async def token_login_handler(request: web.Request) -> web.Response:
     try:
         anon_api_config = APIConfig(
             domain=config["api"]["domain"],
-            endpoint=config["api"]["endpoint"],
+            endpoint=config["api"]["endpoint"][0],
             access_key="",
             secret_key="",  # anonymous session
             user_agent=user_agent,
-            skip_sslcert_validation=not config["api"].get("ssl-verify", True),
+            skip_sslcert_validation=not config["api"]["ssl_verify"],
         )
         assert anon_api_config.is_anonymous
         async with APISession(config=anon_api_config) as api_session:
@@ -610,13 +454,21 @@ async def server_main(
     args: Tuple[Any, ...],
 ) -> AsyncIterator[None]:
     config = args[0]
-    app = web.Application()
+    app = web.Application(middlewares=[decrypt_payload])
     app["config"] = config
+    j2env = jinja2.Environment(
+        extensions=[
+            "ai.backend.web.template.TOMLField",
+            "ai.backend.web.template.TOMLStringListField",
+        ],
+        loader=jinja2.PackageLoader("ai.backend.web", "templates"),
+    )
+    j2env.filters["toml_scalar"] = toml_scalar
+    app["j2env"] = j2env
+
     redis_url = yarl.URL("redis://host").with_host(config["session"]["redis"]["host"]).with_port(
         config["session"]["redis"]["port"]
-    ).with_password(config["session"]["redis"].get("password", None)) / str(
-        config["session"]["redis"].get("db", 0)
-    )  # noqa
+    ).with_password(config["session"]["redis"]["password"]) / str(config["session"]["redis"]["db"])
     keepalive_options = {}
     if hasattr(socket, "TCP_KEEPIDLE"):
         keepalive_options[socket.TCP_KEEPIDLE] = 20
@@ -624,16 +476,19 @@ async def server_main(
         keepalive_options[socket.TCP_KEEPINTVL] = 5
     if hasattr(socket, "TCP_KEEPCNT"):
         keepalive_options[socket.TCP_KEEPCNT] = 3
-    app["redis"] = await aioredis.Redis.from_url(
+    app["redis"] = await Redis.from_url(
         str(redis_url),
         socket_keepalive=True,
         socket_keepalive_options=keepalive_options,
     )
 
-    if pidx == 0 and config["session"].get("flush_on_startup", False):
+    if pidx == 0 and config["session"]["flush_on_startup"]:
         await app["redis"].flushdb()
         log.info("flushed session storage.")
-    redis_storage = RedisStorage(app["redis"], max_age=config["session"]["max_age"])
+    redis_storage = RedisStorage(
+        app["redis"],
+        max_age=config["session"]["max_age"],
+    )
 
     setup_session(app, redis_storage)
     cors_options = {
@@ -659,6 +514,7 @@ async def server_main(
     cors.add(app.router.add_route("GET", "/func/{path:hanati/user}", anon_web_plugin_handler))
     cors.add(app.router.add_route("GET", "/func/{path:cloud/.*$}", anon_web_plugin_handler))
     cors.add(app.router.add_route("POST", "/func/{path:cloud/.*$}", anon_web_plugin_handler))
+    cors.add(app.router.add_route("POST", "/func/{path:saml/.*$}", anon_web_plugin_handler))
     cors.add(app.router.add_route("POST", "/func/{path:auth/signup}", anon_web_plugin_handler))
     cors.add(app.router.add_route("POST", "/func/{path:auth/signout}", web_handler))
     cors.add(app.router.add_route("GET", "/func/{path:stream/kernel/_/events}", web_handler))
@@ -671,7 +527,14 @@ async def server_main(
     cors.add(app.router.add_route("POST", "/func/{path:.*$}", web_handler))
     cors.add(app.router.add_route("PATCH", "/func/{path:.*$}", web_handler))
     cors.add(app.router.add_route("DELETE", "/func/{path:.*$}", web_handler))
+    cors.add(app.router.add_route("GET", "/pipeline/{path:.*$}", web_handler))
+    cors.add(app.router.add_route("PUT", "/pipeline/{path:.*$}", web_handler))
+    cors.add(app.router.add_route("POST", "/pipeline/{path:.*$}", web_handler))
+    cors.add(app.router.add_route("PATCH", "/pipeline/{path:.*$}", web_handler))
+    cors.add(app.router.add_route("DELETE", "/pipeline/{path:.*$}", web_handler))
     if config["service"]["mode"] == "webui":
+        cors.add(app.router.add_route("GET", "/config.ini", config_ini_handler))
+        cors.add(app.router.add_route("GET", "/config.toml", config_toml_handler))
         fallback_handler = console_handler
     elif config["service"]["mode"] == "static":
         fallback_handler = static_handler
@@ -683,11 +546,11 @@ async def server_main(
     app.on_cleanup.append(server_cleanup)
 
     ssl_ctx = None
-    if "ssl-enabled" in config["service"] and config["service"]["ssl-enabled"]:
+    if config["service"]["ssl_enabled"]:
         ssl_ctx = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
         ssl_ctx.load_cert_chain(
-            str(config["service"]["ssl-cert"]),
-            str(config["service"]["ssl-privkey"]),
+            str(config["service"]["ssl_cert"]),
+            str(config["service"]["ssl_privkey"]),
         )
 
     runner = web.AppRunner(app)
@@ -721,7 +584,8 @@ async def server_main(
 )
 @click.option("--debug", is_flag=True, default=False, help="Use more verbose logging.")
 def main(config_path: str, debug: bool) -> None:
-    config = toml.loads(Path(config_path).read_text(encoding="utf-8"))
+    raw_config = tomli.loads(Path(config_path).read_text(encoding="utf-8"))
+    config = config_iv.check(raw_config)
     config["debug"] = debug
     if config["debug"]:
         debugFlag = "DEBUG"
@@ -768,8 +632,9 @@ def main(config_path: str, debug: bool) -> None:
     log.info("runtime: {0}", sys.prefix)
     log_config = logging.getLogger("ai.backend.web.config")
     log_config.debug("debug mode enabled.")
-    print("== Web Server configuration ==")
-    pprint(config)
+    if debug:
+        print("== Web Server configuration ==")
+        pprint(config)
     log.info("serving at {0}:{1}", config["service"]["ip"], config["service"]["port"])
 
     try:
